﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace _.Specification_New {
	class Program {
		static void Main(string[] args) {
			var spec = new Specification<Customer>(c => c.City == "London");
			var customer = new Customer
			{
				City = "London"
			};

			// Simple test - should be true
			//ObjectDumper.Write(spec.IsSatisfiedBy(customer));

			var dataSource = new DataSource( );
			var customers = dataSource.GetCustomerList( );
			var products = dataSource.GetProductList( );


			// Find all customers in London
			//ObjectDumper.Write(customers.SelectSatisfying(spec));

			// Combine two existing specifications to find certain products
			var firstSpec = new Specification<Product>(p => p.UnitPrice > 10);

			var secondSpec = new Specification<Product>(p => p.Category == "Beverages");

			var combinedSpec = firstSpec.AndAlso(secondSpec);

			//ObjectDumper.Write(products.SelectSatisfying(combinedSpec));

			// These lines show how the combined spec is put together
			// using calls to the first and second specs
			//ExpressionDumper.Output(firstSpec.PredicateExpression);
			//ExpressionDumper.Output(secondSpec.PredicateExpression);
			//ExpressionDumper.Output(combinedSpec.PredicateExpression);

			// It is also possible to extend specifications with lambdas directly
			var complexSpec = firstSpec.AndAlso(p => p.Category == "Confections" && p.UnitsInStock > 10);
			//ObjectDumper.Write(products.SelectSatisfying(complexSpec));

			// Finally, of course we can use predefined specs
			//ObjectDumper.Write(customers.SelectSatisfying(new LondonCustomersSpec( )));
		}
	}
}