using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Linq;

namespace _.Specification_New {
	public class Specification<T> : ISpecification<T> {
		public Specification(Expression<Predicate<T>> predicateExpression) {
			this.predicateExpression = predicateExpression;
			this.predicate = predicateExpression.Compile( );
		}

		Expression<Predicate<T>> predicateExpression;
		public Expression<Predicate<T>> PredicateExpression {
			get {
				return predicateExpression;
			}
		}
		Predicate<T> predicate;
		public Predicate<T> Predicate {
			get {
				return predicate;
			}
		}

		#region ISpecification<T> Members

		public bool IsSatisfiedBy(T element) {
			return predicate(element);
		}

		public IEnumerable<T> FilterList(IEnumerable<T> sourceList) {
			// Why can't a Predicate<T> be cast to a Func<T,bool>? Good question.
			// Anyway, hence the stupid wrapper lambda.
			return sourceList.Where<T>((T element) => predicate(element));
		}

		#endregion

		#region Lots of methods to create combinations
		public static Specification<T> AndAlso(Specification<T> specification1, Specification<T> specification2) {
			return new Specification<T>((T element) => specification1.predicate(element) && specification2.predicate(element));
		}

		public static Specification<T> AndAlso(Specification<T> specification, Expression<Predicate<T>> predicateExpression) {
			return AndAlso(specification, new Specification<T>(predicateExpression));
		}

		public Specification<T> AndAlso(Specification<T> otherSpecification) {
			return Specification<T>.AndAlso(this, otherSpecification);
		}

		public Specification<T> AndAlso(Expression<Predicate<T>> predicateExpression) {
			return Specification<T>.AndAlso(this, predicateExpression);
		}

		public static Specification<T> Or(Specification<T> specification1, Specification<T> specification2) {
			return new Specification<T>((T element) => specification1.predicate(element) || specification2.predicate(element));
		}

		public static Specification<T> Or(Specification<T> specification, Expression<Predicate<T>> predicateExpression) {
			return Or(specification, new Specification<T>(predicateExpression));
		}

		public Specification<T> Or(Specification<T> otherSpecification) {
			return Specification<T>.Or(this, otherSpecification);
		}

		public Specification<T> Or(Expression<Predicate<T>> predicateExpression) {
			return Specification<T>.Or(this, predicateExpression);
		}

		public static Specification<T> Not(Specification<T> specification) {
			return new Specification<T>((T element) => !specification.predicate(element));
		}

		public Specification<T> Not( ) {
			return Specification<T>.Not(this);
		}
		#endregion
	}
}
