﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Linq;
using System.Linq.Expressions;
using System.IO;
using System.Reflection;

namespace QBE {
  public partial class FilterForm : Form {
    public FilterForm( ) {
      InitializeComponent( );
    }

    public static Expression<Func<XElement, bool>> GetNewFilter(
      XElement initializationElement,
      Expression<Func<XElement, bool>> oldExpression) {
      var form = new FilterForm( );
      form.Configure(initializationElement, oldExpression);
      if (form.ShowDialog( ) == DialogResult.OK)
        return form.GetExpression( );

      return oldExpression;
    }

    List<TextBox> textBoxes;

    private void Configure(XElement initializationElement,
      Expression<Func<XElement, bool>> oldExpression) {
      var elements = initializationElement.Elements( ).ToList( );
      int rows = elements.Count;
      
      layout.RowCount = rows + 1;
      layout.RowStyles.Clear( );

      textBoxes = new List<TextBox>( );

      for (int row = 0; row < rows; row++) {
        layout.RowStyles.Add(new RowStyle(SizeType.Absolute, 20f));

        var label = new Label( );
        string fieldName = elements[row].Name.LocalName;
        label.Text = fieldName;
        label.TextAlign = ContentAlignment.MiddleLeft;
        layout.Controls.Add(label, 0, row);

        var textBox = new TextBox( );
        textBox.Tag = elements[row];
        textBox.Text = ExtractFieldString(oldExpression, fieldName);
        textBoxes.Add(textBox);
        layout.Controls.Add(textBox, 1, row);
      }

      layout.RowStyles.Add(new RowStyle(SizeType.AutoSize));
    }

    #region Extracting the field string
    private string ExtractFieldString(
      Expression<Func<XElement, bool>> oldExpression, string fieldName) {
      if (oldExpression == null)
        return "";

      try {
        LambdaExpression lambdaExpression = (LambdaExpression) oldExpression;
        return ExtractFieldString(lambdaExpression.Body, fieldName);
      }
      catch {
        // This try/catch block is here to remind you that of course you 
        // shouldn't rely on a precise structure of an expression when parsing it.
        // If the structure was different, I would be getting casting exceptions
        // from my code.

        throw;
      }
    }

    private string ExtractFieldString(Expression expression, string fieldName) {
      MethodCallExpression methodCallExpression = expression as MethodCallExpression;
      if (methodCallExpression != null)
        return ExtractFieldString(methodCallExpression, fieldName);
      else
        return ExtractFieldString((BinaryExpression) expression, fieldName);
    }

    private string ExtractFieldString(MethodCallExpression expression, string fieldName) {
      if (((string) ((ConstantExpression) ((UnaryExpression) ((MethodCallExpression)
              ((MemberExpression) expression.Object).Expression).
              Arguments[0]).Operand).Value) == fieldName) {
        return ((string) ((ConstantExpression) expression.Arguments[0]).Value);
      }

      return "";
    }

    private string ExtractFieldString(BinaryExpression expression, string fieldName) {
      string result = ExtractFieldString(expression.Left, fieldName);
      if (result != String.Empty)
        return result;
      return ExtractFieldString(expression.Right, fieldName);
    }
    #endregion

    private Expression CreateFieldExpression(
      ParameterExpression elementParameter, string fieldName, string fieldContent) {
		
      return Expression.Call(
                Expression.MakeMemberAccess(
                  Expression.Call(elementParameter,
                    typeof(XElement).GetMethod("Element"),
                    new[] { Expression.Convert(
		                Expression.Constant(fieldName),
		                typeof(XName)) }),
                  typeof(XElement).GetProperty("Value")),
                typeof(String).GetMethod("StartsWith", new[] { typeof(string) }),
                new[] { Expression.Constant(fieldContent) });
    }

    private Expression<Func<XElement, bool>> GetExpression( ) {
      var elementParameter = Expression.Parameter(typeof(XElement), "element");

      Expression bodyExpression = null;

      foreach (var textBox in textBoxes) {
        if (textBox.Text != String.Empty) {
          var fieldExpression = CreateFieldExpression(elementParameter,
            ((XElement) textBox.Tag).Name.LocalName, textBox.Text);
          if (bodyExpression == null)
            bodyExpression = fieldExpression;
          else
            bodyExpression = Expression.And(bodyExpression, fieldExpression);
        }
      }

      return bodyExpression != null ?
        Expression.Lambda<Func<XElement, bool>>(
        bodyExpression, new[] { elementParameter }) : null;
    }
  }
}
