﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace SmellsLikeFunctionalProgramming {
	class Program {
		static void Main(string[] args) {
			foreach (MemberInfo memberInfo in GetAllMembers( )) {
				Console.WriteLine(memberInfo.Name);
			}
		}

		// This is the basic procedural approach
		public static List<MemberInfo> GetAllMembers( ) {
			Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies( );
			List<Type> types = new List<Type>( );

			foreach (Assembly assembly in assemblies)
				types.AddRange(assembly.GetTypes( ));

			List<MemberInfo> members = new List<MemberInfo>( );

			foreach (Type type in types)
				members.AddRange(type.GetMembers( ));

			return members;
		}
		
		//    // This version uses Array.ForEach with delegates. This is a somewhat
		//    // functional approach, but it doesn't make use of newer language features.
		//    public static List<MemberInfo> GetAllMembers( ) {
		//      List<MemberInfo> members = new List<MemberInfo>( );
		//
		//      Array.ForEach(AppDomain.CurrentDomain.GetAssemblies( ), delegate(Assembly assembly) {
		//        Array.ForEach(assembly.GetTypes( ), delegate(Type type) {
		//          members.AddRange(type.GetMembers( ));
		//        });
		//      });
		//
		//      return members;
		//    }

		//    // Here we use lambdas instead of delegates
		//    public static List<MemberInfo> GetAllMembers( ) {
		//      List<MemberInfo> members = new List<MemberInfo>( );
		//
		//      Array.ForEach(AppDomain.CurrentDomain.GetAssemblies( ),
		//        assembly => {
		//          Array.ForEach(assembly.GetTypes( ),
		//                     type => { members.AddRange(type.GetMembers( )); });
		//        });
		//
		//      return members;
		//    }

		//      // Finally we use the builtin Filter (SelectMany) functionality for the 
		//      // best result.
		//		public static IEnumerable<MemberInfo> GetAllMembers( ) {
		//			return (AppDomain.CurrentDomain.GetAssemblies( ).SelectMany(
		//			  assembly => assembly.GetTypes( ))).SelectMany(type => type.GetMembers( ));
		//		}

	}
}